import json
import math
import time
from datetime import datetime, timedelta
from typing import Any, Dict, List, Optional, Tuple, Union, Callable

import grpc
from dateutil.parser import isoparse

from neurionpy.auth.interface import Auth
from neurionpy.bank.interface import Bank
from neurionpy.distribution.interface import Distribution
from neurionpy.params.interface import Params
from neurionpy.protos.cosmos.base.query.v1beta1.pagination_pb2 import PageRequest
from neurionpy.staking.interface import Staking
from neurionpy.synapse.client.bank import create_bank_send_msg
from neurionpy.synapse.client.distribution import create_withdraw_delegator_reward
from neurionpy.synapse.client.staking import (
    ValidatorStatus,
    create_delegate_msg,
    create_redelegate_msg,
    create_undelegate_msg,
)
from neurionpy.synapse.client.utils import (
    ensure_timedelta,
    get_paginated,
    prepare_and_broadcast_basic_transaction,
)
from neurionpy.synapse.config import NetworkConfig
from neurionpy.synapse.exceptions import NotFoundError, QueryTimeoutError
from neurionpy.synapse.gas import GasStrategy
from neurionpy.synapse.tx import Transaction, TxState, SigningCfg
from neurionpy.synapse.tx_helpers import MessageLog, SubmittedTx, TxResponse
from neurionpy.synapse.utils.constants import DEFAULT_QUERY_INTERVAL_SECS, DEFAULT_QUERY_TIMEOUT_SECS
from neurionpy.synapse.utils.types import Account, Coin, Validator, StakingPosition, COSMOS_SDK_DEC_COIN_PRECISION, \
    StakingSummary, UnbondingPositions, Block
from neurionpy.synapse.wallet import Wallet
from neurionpy.crypto.address import Address
from neurionpy.protos.cosmos.auth.v1beta1.auth_pb2 import BaseAccount
from neurionpy.protos.cosmos.auth.v1beta1.query_pb2 import QueryAccountRequest
from neurionpy.protos.cosmos.bank.v1beta1.query_pb2 import (
    QueryAllBalancesRequest,
    QueryBalanceRequest,
)
from neurionpy.protos.cosmos.base.tendermint.v1beta1.query_pb2 import (
    GetBlockByHeightRequest,
    GetLatestBlockRequest,
)
from neurionpy.protos.cosmos.crypto.ed25519.keys_pb2 import (  # noqa # pylint: disable=unused-import
    PubKey,
)
from neurionpy.protos.cosmos.distribution.v1beta1.query_pb2 import (
    QueryDelegationRewardsRequest,
)
from neurionpy.protos.cosmos.params.v1beta1.query_pb2 import QueryParamsRequest
from neurionpy.protos.cosmos.staking.v1beta1.query_pb2 import (
    QueryDelegatorDelegationsRequest,
    QueryDelegatorUnbondingDelegationsRequest,
    QueryValidatorsRequest,
)


from neurionpy.protos.cosmos.tx.v1beta1.service_pb2 import (
    BroadcastMode,
    BroadcastTxRequest,
    GetTxRequest,
    SimulateRequest,
)
from neurionpy.tendermint.interface import CosmosBaseTendermint
from neurionpy.tx.interface import TxInterface


class NeurionUtils:
    """Neurion Utils."""

    def __init__(
            self,
            cfg: NetworkConfig,
            auth: Auth,
            param: Params,
            bank: Bank,
            staking: Staking,
            distribution: Distribution,
            txs: TxInterface,
            tendermint: CosmosBaseTendermint,
            gas_strategy: Optional[GasStrategy] = None,
            query_interval_secs: int = DEFAULT_QUERY_INTERVAL_SECS,
            query_timeout_secs: int = DEFAULT_QUERY_TIMEOUT_SECS,
    ):
        """Init ledger client.

        :param cfg: Network configurations
        :param query_interval_secs: int. optional interval int seconds
        :param query_timeout_secs: int. optional interval int seconds
        :param wallet: Wallet, defaults to None
        """
        self._query_interval_secs = query_interval_secs
        self._query_timeout_secs = query_timeout_secs
        self._network_config = cfg
        self._gas_strategy: GasStrategy = gas_strategy
        self._auth = auth
        self._params = param
        self._bank = bank
        self._staking = staking
        self._distribution = distribution
        self._txs = txs
        self._tendermint = tendermint



    @property
    def network_config(self) -> NetworkConfig:
        """Get the network config.

        :return: network config
        """
        return self._network_config

    @property
    def gas_strategy(self) -> GasStrategy:
        """Get gas strategy.

        :return: gas strategy
        """
        return self._gas_strategy

    @gas_strategy.setter
    def gas_strategy(self, strategy: GasStrategy):
        """Set gas strategy.

        :param strategy: strategy
        :raises RuntimeError: Invalid strategy must implement GasStrategy interface
        """
        if not isinstance(strategy, GasStrategy):
            raise RuntimeError("Invalid strategy must implement GasStrategy interface")
        self._gas_strategy = strategy

    def query_account(self, address: Address) -> Account:
        """Query account.

        :param address: address
        :raises RuntimeError: Unexpected account type returned from query
        :return: account details
        """
        request = QueryAccountRequest(address=str(address))
        response = self._auth.Account(request)

        account = BaseAccount()
        if not response.account.Is(BaseAccount.DESCRIPTOR):
            raise RuntimeError("Unexpected account type returned from query")
        response.account.Unpack(account)

        return Account(
            address=address,
            number=account.account_number,
            sequence=account.sequence,
        )

    def query_params(self, subspace: str, key: str) -> Any:
        """Query Prams.

        :param subspace: subspace
        :param key: key
        :return: Query params
        """
        req = QueryParamsRequest(subspace=subspace, key=key)
        resp = self._params.Params(req)
        return json.loads(resp.param.value)

    def query_bank_balance(self, address: Address, denom: Optional[str] = None) -> int:
        """Query bank balance.

        :param address: address
        :param denom: denom, defaults to None
        :return: bank balance
        """
        denom = denom or self.network_config.fee_denomination

        req = QueryBalanceRequest(
            address=str(address),
            denom=denom,
        )

        resp = self._bank.Balance(req)
        assert resp.balance.denom == denom  # sanity check

        return resp.balance.amount

    def query_bank_all_balances(self, address: Address) -> List[Coin]:
        """Query bank all balances.

        :param address: address
        :return: bank all balances
        """
        req = QueryAllBalancesRequest(address=str(address))
        resp = self._bank.AllBalances(req)

        return [Coin(amount=coin.amount, denom=coin.denom) for coin in resp.balances]

    def prepare_and_broadcast_basic_transaction(
            self,
            tx: "Transaction",  # type: ignore # noqa: F821
            sender: "Wallet",  # type: ignore # noqa: F821
            account: Optional["Account"] = None,  # type: ignore # noqa: F821
            gas_limit: Optional[int] = None,
            memo: Optional[str] = None,
    ) -> SubmittedTx:
        """Prepare and broadcast basic transaction.

        :param client: Ledger client
        :param tx: The transaction
        :param sender: The transaction sender
        :param account: The account
        :param gas_limit: The gas limit
        :param memo: Transaction memo, defaults to None

        :return: broadcast transaction
        """
        # the sender must be a valid wallet
        if not isinstance(sender, Wallet):
            raise RuntimeError("Invalid sender, must be a Wallet instance")
        # query the account information for the sender
        if account is None:
            account = self.query_account(sender.address())

        if gas_limit is not None:
            # simply build the fee from the provided gas limit
            fee = self.estimate_fee_from_gas(gas_limit)
        else:
            # we need to build up a representative transaction so that we can accurately simulate it
            tx.seal(
                SigningCfg.direct(sender.public_key(), account.sequence),
                fee="",
                gas_limit=0,
                memo=memo,
            )
            tx.sign(sender.signer(), self.network_config.chain_id, account.number)
            tx.complete()

            # simulate the gas and fee for the transaction
            gas_limit, fee = self.estimate_gas_and_fee_for_tx(tx)

        # finally, build the final transaction that will be executed with the correct gas and fee values
        tx.seal(
            SigningCfg.direct(sender.public_key(), account.sequence),
            fee=fee,
            gas_limit=gas_limit,
            memo=memo,
        )
        tx.sign(sender.signer(), self.network_config.chain_id, account.number)
        tx.complete()

        return self.broadcast_tx(tx)

    def ensure_timedelta(interval: Union[int, float, timedelta]) -> timedelta:
        """
        Return timedelta for interval.

        :param interval: timedelta or seconds in int or float

        :return: timedelta
        """
        return interval if isinstance(interval, timedelta) else timedelta(seconds=interval)

    DEFAULT_PER_PAGE_LIMIT = None

    def get_paginated(
            initial_request: Any,
            request_method: Callable,
            pages_limit: int = 0,
            per_page_limit: Optional[int] = DEFAULT_PER_PAGE_LIMIT,
    ) -> List[Any]:
        """
        Get pages for specific request.

        :param initial_request: request supports pagination
        :param request_method: function to perform request
        :param pages_limit: max number of pages to return. default - 0 unlimited
        :param per_page_limit: Optional int: amount of records per one page. default is None, determined by server

        :return: List of responses
        """
        pages: List[Any] = []
        pagination = PageRequest(limit=per_page_limit)

        while pagination and (len(pages) < pages_limit or pages_limit == 0):
            request = initial_request.__class__()
            request.CopyFrom(initial_request)
            request.pagination.CopyFrom(pagination)

            resp = request_method(request)

            pages.append(resp)

            pagination = None

            if resp.pagination.next_key:
                pagination = PageRequest(limit=per_page_limit, key=resp.pagination.next_key)
        return pages

    def send_tokens(
            self,
            destination: Address,
            amount: int,
            denom: str,
            sender: Wallet,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Send tokens.

        :param destination: destination address
        :param amount: amount
        :param denom: denom
        :param sender: sender
        :param memo: memo, defaults to None
        :param gas_limit: gas limit, defaults to None
        :return: prepare and broadcast the transaction and transaction details
        """
        # build up the store transaction
        tx = Transaction()
        tx.add_message(
            create_bank_send_msg(sender.address(), destination, amount, denom)
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def query_validators(
            self, status: Optional[ValidatorStatus] = None
    ) -> List[Validator]:
        """Query validators.

        :param status: validator status, defaults to None
        :return: List of validators
        """
        filtered_status = status or ValidatorStatus.BONDED

        req = QueryValidatorsRequest()
        if filtered_status != ValidatorStatus.UNSPECIFIED:
            req.status = filtered_status.value

        resp = self._staking.Validators(req)

        validators: List[Validator] = []
        for validator in resp.validators:
            validators.append(
                Validator(
                    address=Address(validator.operator_address),
                    tokens=int(float(validator.tokens)),
                    moniker=str(validator.description.moniker),
                    status=ValidatorStatus.from_proto(validator.status),
                )
            )
        return validators

    def query_staking_summary(self, address: Address) -> StakingSummary:
        """Query staking summary.

        :param address: address
        :return: staking summary
        """
        current_positions: List[StakingPosition] = []

        req = QueryDelegatorDelegationsRequest(delegator_addr=str(address))

        for resp in get_paginated(
                req, self._staking.DelegatorDelegations, per_page_limit=1
        ):
            for item in resp.delegation_responses:
                req = QueryDelegationRewardsRequest(
                    delegator_address=str(address),
                    validator_address=str(item.delegation.validator_address),
                )
                rewards_resp = self._distribution.DelegationRewards(req)

                stake_reward = 0
                for reward in rewards_resp.rewards:
                    if reward.denom == self.network_config.staking_denomination:
                        stake_reward = (
                                int(float(reward.amount)) // COSMOS_SDK_DEC_COIN_PRECISION
                        )
                        break

                current_positions.append(
                    StakingPosition(
                        validator=Address(item.delegation.validator_address),
                        amount=int(float(item.balance.amount)),
                        reward=stake_reward,
                    )
                )

        unbonding_summary: Dict[str, int] = {}
        req = QueryDelegatorUnbondingDelegationsRequest(delegator_addr=str(address))

        for resp in get_paginated(req, self._staking.DelegatorUnbondingDelegations):
            for item in resp.unbonding_responses:
                validator = str(item.validator_address)
                total_unbonding = unbonding_summary.get(validator, 0)

                for entry in item.entries:
                    total_unbonding += int(float(entry.balance))

                unbonding_summary[validator] = total_unbonding

        # build the final list of unbonding positions
        unbonding_positions: List[UnbondingPositions] = []
        for validator, total_unbonding in unbonding_summary.items():
            unbonding_positions.append(
                UnbondingPositions(
                    validator=Address(validator),
                    amount=total_unbonding,
                )
            )

        return StakingSummary(
            current_positions=current_positions, unbonding_positions=unbonding_positions
        )

    def delegate_tokens(
            self,
            validator: Address,
            amount: int,
            sender: Wallet,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Delegate tokens.

        :param validator: validator address
        :param amount: amount
        :param sender: sender
        :param memo: memo, defaults to None
        :param gas_limit: gas limit, defaults to None
        :return: prepare and broadcast the transaction and transaction details
        """
        tx = Transaction()
        tx.add_message(
            create_delegate_msg(
                sender.address(),
                validator,
                amount,
                self.network_config.staking_denomination,
            )
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def redelegate_tokens(
            self,
            current_validator: Address,
            next_validator: Address,
            amount: int,
            sender: Wallet,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Redelegate tokens.

        :param current_validator: current validator address
        :param next_validator: next validator address
        :param amount: amount
        :param sender: sender
        :param memo: memo, defaults to None
        :param gas_limit: gas limit, defaults to None
        :return: prepare and broadcast the transaction and transaction details
        """
        tx = Transaction()
        tx.add_message(
            create_redelegate_msg(
                sender.address(),
                current_validator,
                next_validator,
                amount,
                self.network_config.staking_denomination,
            )
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def undelegate_tokens(
            self,
            validator: Address,
            amount: int,
            sender: Wallet,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Undelegate tokens.

        :param validator: validator
        :param amount: amount
        :param sender: sender
        :param memo: memo, defaults to None
        :param gas_limit: gas limit, defaults to None
        :return: prepare and broadcast the transaction and transaction details
        """
        tx = Transaction()
        tx.add_message(
            create_undelegate_msg(
                sender.address(),
                validator,
                amount,
                self.network_config.staking_denomination,
            )
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def claim_rewards(
            self,
            validator: Address,
            sender: Wallet,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """claim rewards.

        :param validator: validator
        :param sender: sender
        :param memo: memo, defaults to None
        :param gas_limit: gas limit, defaults to None
        :return: prepare and broadcast the transaction and transaction details
        """
        tx = Transaction()
        tx.add_message(create_withdraw_delegator_reward(sender.address(), validator))

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def estimate_gas_for_tx(self, tx: Transaction) -> int:
        """Estimate gas for transaction.

        :param tx: transaction
        :return: Estimated gas for transaction
        """
        return self._gas_strategy.estimate_gas(tx)

    def estimate_fee_from_gas(self, gas_limit: int) -> str:
        """Estimate fee from gas.

        :param gas_limit: gas limit
        :return: Estimated fee for transaction
        """
        fee = math.ceil(gas_limit * self.network_config.fee_minimum_gas_price)
        return f"{fee}{self.network_config.fee_denomination}"

    def estimate_gas_and_fee_for_tx(self, tx: Transaction) -> Tuple[int, str]:
        """Estimate gas and fee for transaction.

        :param tx: transaction
        :return: estimate gas, fee for transaction
        """
        gas_estimate = self.estimate_gas_for_tx(tx)
        fee = self.estimate_fee_from_gas(gas_estimate)
        return gas_estimate, fee

    def wait_for_query_tx(
            self,
            tx_hash: str,
            timeout: Optional[timedelta] = None,
            poll_period: Optional[timedelta] = None,
    ) -> TxResponse:
        """Wait for query transaction.

        :param tx_hash: transaction hash
        :param timeout: timeout, defaults to None
        :param poll_period: poll_period, defaults to None

        :raises QueryTimeoutError: timeout

        :return: transaction response
        """
        timeout = (
            ensure_timedelta(timeout)
            if timeout
            else timedelta(seconds=self._query_timeout_secs)
        )
        poll_period = (
            ensure_timedelta(poll_period)
            if poll_period
            else timedelta(seconds=self._query_interval_secs)
        )

        start = datetime.now()
        while True:
            try:
                return self.query_tx(tx_hash)
            except NotFoundError:
                pass

            delta = datetime.now() - start
            if delta >= timeout:
                raise QueryTimeoutError()

            time.sleep(poll_period.total_seconds())

    def query_tx(self, tx_hash: str) -> TxResponse:
        """query transaction.

        :param tx_hash: transaction hash
        :raises NotFoundError: Tx details not found
        :raises grpc.RpcError: RPC connection issue
        :return: query response
        """
        req = GetTxRequest(hash=tx_hash)
        try:
            resp = self._txs.GetTx(req)
        except grpc.RpcError as e:
            details = e.details()
            if "not found" in details:
                raise NotFoundError() from e
            raise
        except RuntimeError as e:
            details = str(e)
            if "tx" in details and "not found" in details:
                raise NotFoundError() from e
            raise

        return self._parse_tx_response(resp.tx_response)

    @staticmethod
    def _parse_tx_response(tx_response: Any) -> TxResponse:
        # parse the transaction logs
        logs = []
        for log_data in tx_response.logs:
            events = {}
            for event in log_data.events:
                events[event.type] = {a.key: a.value for a in event.attributes}
            logs.append(
                MessageLog(
                    index=int(log_data.msg_index), log=log_data.msg_index, events=events
                )
            )

        # parse the transaction events
        events = {}
        for event in tx_response.events:
            event_data = events.get(event.type, {})
            for attribute in event.attributes:
                event_data[attribute.key.decode()] = attribute.value.decode()
            events[event.type] = event_data

        timestamp = None
        if tx_response.timestamp:
            timestamp = isoparse(tx_response.timestamp)

        return TxResponse(
            hash=str(tx_response.txhash),
            height=int(tx_response.height),
            code=int(tx_response.code),
            gas_wanted=int(tx_response.gas_wanted),
            gas_used=int(tx_response.gas_used),
            raw_log=str(tx_response.raw_log),
            logs=logs,
            events=events,
            timestamp=timestamp,
        )

    def simulate_tx(self, tx: Transaction) -> int:
        """simulate transaction.

        :param tx: transaction
        :raises RuntimeError: Unable to simulate non final transaction
        :return: gas used in transaction
        """
        if tx.state != TxState.Final:
            raise RuntimeError("Unable to simulate non final transaction")

        req = SimulateRequest(tx=tx.tx)
        resp = self._txs.Simulate(req)

        return int(resp.gas_info.gas_used)

    def broadcast_tx(self, tx: Transaction) -> SubmittedTx:
        """Broadcast transaction.

        :param tx: transaction
        :return: Submitted transaction
        """
        # create the broadcast request
        broadcast_req = BroadcastTxRequest(
            tx_bytes=tx.tx.SerializeToString(), mode=BroadcastMode.BROADCAST_MODE_SYNC
        )

        # broadcast the transaction
        resp = self._txs.BroadcastTx(broadcast_req)
        tx_digest = resp.tx_response.txhash

        # check that the response is successful
        initial_tx_response = self._parse_tx_response(resp.tx_response)
        initial_tx_response.ensure_successful()

        return SubmittedTx(self, tx_digest)

    def query_latest_block(self) -> Block:
        """Query the latest block.

        :return: latest block
        """
        req = GetLatestBlockRequest()
        resp = self._tendermint.GetLatestBlock(req)
        return Block.from_proto(resp.block)

    def query_block(self, height: int) -> Block:
        """Query the block.

        :param height: block height
        :return: block
        """
        req = GetBlockByHeightRequest(height=height)
        resp = self._tendermint.GetBlockByHeight(req)
        return Block.from_proto(resp.block)

    def query_height(self) -> int:
        """Query the latest block height.

        :return: latest block height
        """
        return self.query_latest_block().height

    def query_chain_id(self) -> str:
        """Query the chain id.

        :return: chain id
        """
        return self.query_latest_block().chain_id


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cometbft/consensus/v1/types.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'cometbft/consensus/v1/types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cometbft.libs.bits.v1 import types_pb2 as cometbft_dot_libs_dot_bits_dot_v1_dot_types__pb2
from cometbft.types.v1 import types_pb2 as cometbft_dot_types_dot_v1_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!cometbft/consensus/v1/types.proto\x12\x15\x63ometbft.consensus.v1\x1a\x14gogoproto/gogo.proto\x1a!cometbft/libs/bits/v1/types.proto\x1a\x1d\x63ometbft/types/v1/types.proto\"x\n\x0cNewRoundStep\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05round\x18\x02 \x01(\x05\x12\x0c\n\x04step\x18\x03 \x01(\r\x12 \n\x18seconds_since_start_time\x18\x04 \x01(\x03\x12\x19\n\x11last_commit_round\x18\x05 \x01(\x05\"\xbe\x01\n\rNewValidBlock\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05round\x18\x02 \x01(\x05\x12\x45\n\x15\x62lock_part_set_header\x18\x03 \x01(\x0b\x32 .cometbft.types.v1.PartSetHeaderB\x04\xc8\xde\x1f\x00\x12\x34\n\x0b\x62lock_parts\x18\x04 \x01(\x0b\x32\x1f.cometbft.libs.bits.v1.BitArray\x12\x11\n\tis_commit\x18\x05 \x01(\x08\"?\n\x08Proposal\x12\x33\n\x08proposal\x18\x01 \x01(\x0b\x32\x1b.cometbft.types.v1.ProposalB\x04\xc8\xde\x1f\x00\"v\n\x0bProposalPOL\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\x1a\n\x12proposal_pol_round\x18\x02 \x01(\x05\x12;\n\x0cproposal_pol\x18\x03 \x01(\x0b\x32\x1f.cometbft.libs.bits.v1.BitArrayB\x04\xc8\xde\x1f\x00\"W\n\tBlockPart\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05round\x18\x02 \x01(\x05\x12+\n\x04part\x18\x03 \x01(\x0b\x32\x17.cometbft.types.v1.PartB\x04\xc8\xde\x1f\x00\"-\n\x04Vote\x12%\n\x04vote\x18\x01 \x01(\x0b\x32\x17.cometbft.types.v1.Vote\"g\n\x07HasVote\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05round\x18\x02 \x01(\x05\x12.\n\x04type\x18\x03 \x01(\x0e\x32 .cometbft.types.v1.SignedMsgType\x12\r\n\x05index\x18\x04 \x01(\x05\"\x9c\x01\n\x0cVoteSetMaj23\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05round\x18\x02 \x01(\x05\x12.\n\x04type\x18\x03 \x01(\x0e\x32 .cometbft.types.v1.SignedMsgType\x12=\n\x08\x62lock_id\x18\x04 \x01(\x0b\x32\x1a.cometbft.types.v1.BlockIDB\x0f\xc8\xde\x1f\x00\xe2\xde\x1f\x07\x42lockID\"\xd1\x01\n\x0bVoteSetBits\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05round\x18\x02 \x01(\x05\x12.\n\x04type\x18\x03 \x01(\x0e\x32 .cometbft.types.v1.SignedMsgType\x12=\n\x08\x62lock_id\x18\x04 \x01(\x0b\x32\x1a.cometbft.types.v1.BlockIDB\x0f\xc8\xde\x1f\x00\xe2\xde\x1f\x07\x42lockID\x12\x34\n\x05votes\x18\x05 \x01(\x0b\x32\x1f.cometbft.libs.bits.v1.BitArrayB\x04\xc8\xde\x1f\x00\"D\n\x14HasProposalBlockPart\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05round\x18\x02 \x01(\x05\x12\r\n\x05index\x18\x03 \x01(\x05\"\xe6\x04\n\x07Message\x12=\n\x0enew_round_step\x18\x01 \x01(\x0b\x32#.cometbft.consensus.v1.NewRoundStepH\x00\x12?\n\x0fnew_valid_block\x18\x02 \x01(\x0b\x32$.cometbft.consensus.v1.NewValidBlockH\x00\x12\x33\n\x08proposal\x18\x03 \x01(\x0b\x32\x1f.cometbft.consensus.v1.ProposalH\x00\x12:\n\x0cproposal_pol\x18\x04 \x01(\x0b\x32\".cometbft.consensus.v1.ProposalPOLH\x00\x12\x36\n\nblock_part\x18\x05 \x01(\x0b\x32 .cometbft.consensus.v1.BlockPartH\x00\x12+\n\x04vote\x18\x06 \x01(\x0b\x32\x1b.cometbft.consensus.v1.VoteH\x00\x12\x32\n\x08has_vote\x18\x07 \x01(\x0b\x32\x1e.cometbft.consensus.v1.HasVoteH\x00\x12=\n\x0evote_set_maj23\x18\x08 \x01(\x0b\x32#.cometbft.consensus.v1.VoteSetMaj23H\x00\x12;\n\rvote_set_bits\x18\t \x01(\x0b\x32\".cometbft.consensus.v1.VoteSetBitsH\x00\x12N\n\x17has_proposal_block_part\x18\n \x01(\x0b\x32+.cometbft.consensus.v1.HasProposalBlockPartH\x00\x42\x05\n\x03sumB8Z6github.com/cometbft/cometbft/api/cometbft/consensus/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cometbft.consensus.v1.types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z6github.com/cometbft/cometbft/api/cometbft/consensus/v1'
  _globals['_NEWVALIDBLOCK'].fields_by_name['block_part_set_header']._loaded_options = None
  _globals['_NEWVALIDBLOCK'].fields_by_name['block_part_set_header']._serialized_options = b'\310\336\037\000'
  _globals['_PROPOSAL'].fields_by_name['proposal']._loaded_options = None
  _globals['_PROPOSAL'].fields_by_name['proposal']._serialized_options = b'\310\336\037\000'
  _globals['_PROPOSALPOL'].fields_by_name['proposal_pol']._loaded_options = None
  _globals['_PROPOSALPOL'].fields_by_name['proposal_pol']._serialized_options = b'\310\336\037\000'
  _globals['_BLOCKPART'].fields_by_name['part']._loaded_options = None
  _globals['_BLOCKPART'].fields_by_name['part']._serialized_options = b'\310\336\037\000'
  _globals['_VOTESETMAJ23'].fields_by_name['block_id']._loaded_options = None
  _globals['_VOTESETMAJ23'].fields_by_name['block_id']._serialized_options = b'\310\336\037\000\342\336\037\007BlockID'
  _globals['_VOTESETBITS'].fields_by_name['block_id']._loaded_options = None
  _globals['_VOTESETBITS'].fields_by_name['block_id']._serialized_options = b'\310\336\037\000\342\336\037\007BlockID'
  _globals['_VOTESETBITS'].fields_by_name['votes']._loaded_options = None
  _globals['_VOTESETBITS'].fields_by_name['votes']._serialized_options = b'\310\336\037\000'
  _globals['_NEWROUNDSTEP']._serialized_start=148
  _globals['_NEWROUNDSTEP']._serialized_end=268
  _globals['_NEWVALIDBLOCK']._serialized_start=271
  _globals['_NEWVALIDBLOCK']._serialized_end=461
  _globals['_PROPOSAL']._serialized_start=463
  _globals['_PROPOSAL']._serialized_end=526
  _globals['_PROPOSALPOL']._serialized_start=528
  _globals['_PROPOSALPOL']._serialized_end=646
  _globals['_BLOCKPART']._serialized_start=648
  _globals['_BLOCKPART']._serialized_end=735
  _globals['_VOTE']._serialized_start=737
  _globals['_VOTE']._serialized_end=782
  _globals['_HASVOTE']._serialized_start=784
  _globals['_HASVOTE']._serialized_end=887
  _globals['_VOTESETMAJ23']._serialized_start=890
  _globals['_VOTESETMAJ23']._serialized_end=1046
  _globals['_VOTESETBITS']._serialized_start=1049
  _globals['_VOTESETBITS']._serialized_end=1258
  _globals['_HASPROPOSALBLOCKPART']._serialized_start=1260
  _globals['_HASPROPOSALBLOCKPART']._serialized_end=1328
  _globals['_MESSAGE']._serialized_start=1331
  _globals['_MESSAGE']._serialized_end=1945
# @@protoc_insertion_point(module_scope)



from gogoproto import gogo_pb2 as _gogo_pb2
from cosmos.base.query.v1beta1 import pagination_pb2 as _pagination_pb2
from ibc.applications.transfer.v1 import transfer_pb2 as _transfer_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueryDenomTraceRequest(_message.Message):
    __slots__ = ("hash",)
    HASH_FIELD_NUMBER: _ClassVar[int]
    hash: str
    def __init__(self, hash: _Optional[str] = ...) -> None: ...

class QueryDenomTraceResponse(_message.Message):
    __slots__ = ("denom_trace",)
    DENOM_TRACE_FIELD_NUMBER: _ClassVar[int]
    denom_trace: _transfer_pb2.DenomTrace
    def __init__(self, denom_trace: _Optional[_Union[_transfer_pb2.DenomTrace, _Mapping]] = ...) -> None: ...

class QueryDenomTracesRequest(_message.Message):
    __slots__ = ("pagination",)
    PAGINATION_FIELD_NUMBER: _ClassVar[int]
    pagination: _pagination_pb2.PageRequest
    def __init__(self, pagination: _Optional[_Union[_pagination_pb2.PageRequest, _Mapping]] = ...) -> None: ...

class QueryDenomTracesResponse(_message.Message):
    __slots__ = ("denom_traces", "pagination")
    DENOM_TRACES_FIELD_NUMBER: _ClassVar[int]
    PAGINATION_FIELD_NUMBER: _ClassVar[int]
    denom_traces: _containers.RepeatedCompositeFieldContainer[_transfer_pb2.DenomTrace]
    pagination: _pagination_pb2.PageResponse
    def __init__(self, denom_traces: _Optional[_Iterable[_Union[_transfer_pb2.DenomTrace, _Mapping]]] = ..., pagination: _Optional[_Union[_pagination_pb2.PageResponse, _Mapping]] = ...) -> None: ...

class QueryParamsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class QueryParamsResponse(_message.Message):
    __slots__ = ("params",)
    PARAMS_FIELD_NUMBER: _ClassVar[int]
    params: _transfer_pb2.Params
    def __init__(self, params: _Optional[_Union[_transfer_pb2.Params, _Mapping]] = ...) -> None: ...

class QueryDenomHashRequest(_message.Message):
    __slots__ = ("trace",)
    TRACE_FIELD_NUMBER: _ClassVar[int]
    trace: str
    def __init__(self, trace: _Optional[str] = ...) -> None: ...

class QueryDenomHashResponse(_message.Message):
    __slots__ = ("hash",)
    HASH_FIELD_NUMBER: _ClassVar[int]
    hash: str
    def __init__(self, hash: _Optional[str] = ...) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from neurion.sanctum import tx_pb2 as neurion_dot_sanctum_dot_tx__pb2

GRPC_GENERATED_VERSION = '1.70.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in neurion/sanctum/tx_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class MsgStub(object):
    """Msg defines the Msg service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.UpdateParams = channel.unary_unary(
                '/neurion.sanctum.Msg/UpdateParams',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgUpdateParams.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgUpdateParamsResponse.FromString,
                _registered_method=True)
        self.SubmitDatasetApplication = channel.unary_unary(
                '/neurion.sanctum.Msg/SubmitDatasetApplication',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgSubmitDatasetApplication.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgSubmitDatasetApplicationResponse.FromString,
                _registered_method=True)
        self.ApproveApplication = channel.unary_unary(
                '/neurion.sanctum.Msg/ApproveApplication',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveApplication.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveApplicationResponse.FromString,
                _registered_method=True)
        self.RejectApplication = channel.unary_unary(
                '/neurion.sanctum.Msg/RejectApplication',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectApplication.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectApplicationResponse.FromString,
                _registered_method=True)
        self.DisclaimDataset = channel.unary_unary(
                '/neurion.sanctum.Msg/DisclaimDataset',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisclaimDataset.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisclaimDatasetResponse.FromString,
                _registered_method=True)
        self.RequestToUseDataset = channel.unary_unary(
                '/neurion.sanctum.Msg/RequestToUseDataset',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRequestToUseDataset.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRequestToUseDatasetResponse.FromString,
                _registered_method=True)
        self.CancelDatasetUsageRequest = channel.unary_unary(
                '/neurion.sanctum.Msg/CancelDatasetUsageRequest',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgCancelDatasetUsageRequest.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgCancelDatasetUsageRequestResponse.FromString,
                _registered_method=True)
        self.RejectDatasetUsageRequest = channel.unary_unary(
                '/neurion.sanctum.Msg/RejectDatasetUsageRequest',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDatasetUsageRequest.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDatasetUsageRequestResponse.FromString,
                _registered_method=True)
        self.ApproveDatasetUsageRequest = channel.unary_unary(
                '/neurion.sanctum.Msg/ApproveDatasetUsageRequest',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDatasetUsageRequest.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDatasetUsageRequestResponse.FromString,
                _registered_method=True)
        self.AddProcessor = channel.unary_unary(
                '/neurion.sanctum.Msg/AddProcessor',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgAddProcessor.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgAddProcessorResponse.FromString,
                _registered_method=True)
        self.RemoveProcessor = channel.unary_unary(
                '/neurion.sanctum.Msg/RemoveProcessor',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRemoveProcessor.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRemoveProcessorResponse.FromString,
                _registered_method=True)
        self.ProcessDatasetUsageRequest = channel.unary_unary(
                '/neurion.sanctum.Msg/ProcessDatasetUsageRequest',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgProcessDatasetUsageRequest.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgProcessDatasetUsageRequestResponse.FromString,
                _registered_method=True)
        self.FinishDatasetUsageRequest = channel.unary_unary(
                '/neurion.sanctum.Msg/FinishDatasetUsageRequest',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgFinishDatasetUsageRequest.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgFinishDatasetUsageRequestResponse.FromString,
                _registered_method=True)
        self.DisputeDatasetUsageRequest = channel.unary_unary(
                '/neurion.sanctum.Msg/DisputeDatasetUsageRequest',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisputeDatasetUsageRequest.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisputeDatasetUsageRequestResponse.FromString,
                _registered_method=True)
        self.ApproveDispute = channel.unary_unary(
                '/neurion.sanctum.Msg/ApproveDispute',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDispute.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDisputeResponse.FromString,
                _registered_method=True)
        self.RejectDispute = channel.unary_unary(
                '/neurion.sanctum.Msg/RejectDispute',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDispute.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDisputeResponse.FromString,
                _registered_method=True)
        self.StakeToSanctum = channel.unary_unary(
                '/neurion.sanctum.Msg/StakeToSanctum',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgStakeToSanctum.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgStakeToSanctumResponse.FromString,
                _registered_method=True)
        self.UnstakeFromSanctum = channel.unary_unary(
                '/neurion.sanctum.Msg/UnstakeFromSanctum',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgUnstakeFromSanctum.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgUnstakeFromSanctumResponse.FromString,
                _registered_method=True)
        self.ClaimReward = channel.unary_unary(
                '/neurion.sanctum.Msg/ClaimReward',
                request_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgClaimReward.SerializeToString,
                response_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgClaimRewardResponse.FromString,
                _registered_method=True)


class MsgServicer(object):
    """Msg defines the Msg service.
    """

    def UpdateParams(self, request, context):
        """UpdateParams defines a (governance) operation for updating the module
        parameters. The authority defaults to the x/gov module account.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubmitDatasetApplication(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApproveApplication(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RejectApplication(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DisclaimDataset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RequestToUseDataset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CancelDatasetUsageRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RejectDatasetUsageRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApproveDatasetUsageRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddProcessor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveProcessor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ProcessDatasetUsageRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FinishDatasetUsageRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DisputeDatasetUsageRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApproveDispute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RejectDispute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StakeToSanctum(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnstakeFromSanctum(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClaimReward(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MsgServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'UpdateParams': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateParams,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgUpdateParams.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgUpdateParamsResponse.SerializeToString,
            ),
            'SubmitDatasetApplication': grpc.unary_unary_rpc_method_handler(
                    servicer.SubmitDatasetApplication,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgSubmitDatasetApplication.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgSubmitDatasetApplicationResponse.SerializeToString,
            ),
            'ApproveApplication': grpc.unary_unary_rpc_method_handler(
                    servicer.ApproveApplication,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveApplication.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveApplicationResponse.SerializeToString,
            ),
            'RejectApplication': grpc.unary_unary_rpc_method_handler(
                    servicer.RejectApplication,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectApplication.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectApplicationResponse.SerializeToString,
            ),
            'DisclaimDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.DisclaimDataset,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisclaimDataset.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisclaimDatasetResponse.SerializeToString,
            ),
            'RequestToUseDataset': grpc.unary_unary_rpc_method_handler(
                    servicer.RequestToUseDataset,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRequestToUseDataset.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRequestToUseDatasetResponse.SerializeToString,
            ),
            'CancelDatasetUsageRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.CancelDatasetUsageRequest,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgCancelDatasetUsageRequest.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgCancelDatasetUsageRequestResponse.SerializeToString,
            ),
            'RejectDatasetUsageRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.RejectDatasetUsageRequest,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDatasetUsageRequest.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDatasetUsageRequestResponse.SerializeToString,
            ),
            'ApproveDatasetUsageRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.ApproveDatasetUsageRequest,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDatasetUsageRequest.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDatasetUsageRequestResponse.SerializeToString,
            ),
            'AddProcessor': grpc.unary_unary_rpc_method_handler(
                    servicer.AddProcessor,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgAddProcessor.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgAddProcessorResponse.SerializeToString,
            ),
            'RemoveProcessor': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveProcessor,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRemoveProcessor.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRemoveProcessorResponse.SerializeToString,
            ),
            'ProcessDatasetUsageRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.ProcessDatasetUsageRequest,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgProcessDatasetUsageRequest.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgProcessDatasetUsageRequestResponse.SerializeToString,
            ),
            'FinishDatasetUsageRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.FinishDatasetUsageRequest,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgFinishDatasetUsageRequest.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgFinishDatasetUsageRequestResponse.SerializeToString,
            ),
            'DisputeDatasetUsageRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.DisputeDatasetUsageRequest,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisputeDatasetUsageRequest.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgDisputeDatasetUsageRequestResponse.SerializeToString,
            ),
            'ApproveDispute': grpc.unary_unary_rpc_method_handler(
                    servicer.ApproveDispute,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDispute.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgApproveDisputeResponse.SerializeToString,
            ),
            'RejectDispute': grpc.unary_unary_rpc_method_handler(
                    servicer.RejectDispute,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDispute.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgRejectDisputeResponse.SerializeToString,
            ),
            'StakeToSanctum': grpc.unary_unary_rpc_method_handler(
                    servicer.StakeToSanctum,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgStakeToSanctum.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgStakeToSanctumResponse.SerializeToString,
            ),
            'UnstakeFromSanctum': grpc.unary_unary_rpc_method_handler(
                    servicer.UnstakeFromSanctum,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgUnstakeFromSanctum.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgUnstakeFromSanctumResponse.SerializeToString,
            ),
            'ClaimReward': grpc.unary_unary_rpc_method_handler(
                    servicer.ClaimReward,
                    request_deserializer=neurion_dot_sanctum_dot_tx__pb2.MsgClaimReward.FromString,
                    response_serializer=neurion_dot_sanctum_dot_tx__pb2.MsgClaimRewardResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'neurion.sanctum.Msg', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('neurion.sanctum.Msg', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class Msg(object):
    """Msg defines the Msg service.
    """

    @staticmethod
    def UpdateParams(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/UpdateParams',
            neurion_dot_sanctum_dot_tx__pb2.MsgUpdateParams.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgUpdateParamsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SubmitDatasetApplication(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/SubmitDatasetApplication',
            neurion_dot_sanctum_dot_tx__pb2.MsgSubmitDatasetApplication.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgSubmitDatasetApplicationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ApproveApplication(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/ApproveApplication',
            neurion_dot_sanctum_dot_tx__pb2.MsgApproveApplication.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgApproveApplicationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RejectApplication(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/RejectApplication',
            neurion_dot_sanctum_dot_tx__pb2.MsgRejectApplication.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgRejectApplicationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DisclaimDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/DisclaimDataset',
            neurion_dot_sanctum_dot_tx__pb2.MsgDisclaimDataset.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgDisclaimDatasetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RequestToUseDataset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/RequestToUseDataset',
            neurion_dot_sanctum_dot_tx__pb2.MsgRequestToUseDataset.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgRequestToUseDatasetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CancelDatasetUsageRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/CancelDatasetUsageRequest',
            neurion_dot_sanctum_dot_tx__pb2.MsgCancelDatasetUsageRequest.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgCancelDatasetUsageRequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RejectDatasetUsageRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/RejectDatasetUsageRequest',
            neurion_dot_sanctum_dot_tx__pb2.MsgRejectDatasetUsageRequest.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgRejectDatasetUsageRequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ApproveDatasetUsageRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/ApproveDatasetUsageRequest',
            neurion_dot_sanctum_dot_tx__pb2.MsgApproveDatasetUsageRequest.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgApproveDatasetUsageRequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AddProcessor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/AddProcessor',
            neurion_dot_sanctum_dot_tx__pb2.MsgAddProcessor.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgAddProcessorResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RemoveProcessor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/RemoveProcessor',
            neurion_dot_sanctum_dot_tx__pb2.MsgRemoveProcessor.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgRemoveProcessorResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ProcessDatasetUsageRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/ProcessDatasetUsageRequest',
            neurion_dot_sanctum_dot_tx__pb2.MsgProcessDatasetUsageRequest.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgProcessDatasetUsageRequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def FinishDatasetUsageRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/FinishDatasetUsageRequest',
            neurion_dot_sanctum_dot_tx__pb2.MsgFinishDatasetUsageRequest.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgFinishDatasetUsageRequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DisputeDatasetUsageRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/DisputeDatasetUsageRequest',
            neurion_dot_sanctum_dot_tx__pb2.MsgDisputeDatasetUsageRequest.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgDisputeDatasetUsageRequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ApproveDispute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/ApproveDispute',
            neurion_dot_sanctum_dot_tx__pb2.MsgApproveDispute.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgApproveDisputeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RejectDispute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/RejectDispute',
            neurion_dot_sanctum_dot_tx__pb2.MsgRejectDispute.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgRejectDisputeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StakeToSanctum(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/StakeToSanctum',
            neurion_dot_sanctum_dot_tx__pb2.MsgStakeToSanctum.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgStakeToSanctumResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UnstakeFromSanctum(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/UnstakeFromSanctum',
            neurion_dot_sanctum_dot_tx__pb2.MsgUnstakeFromSanctum.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgUnstakeFromSanctumResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ClaimReward(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/neurion.sanctum.Msg/ClaimReward',
            neurion_dot_sanctum_dot_tx__pb2.MsgClaimReward.SerializeToString,
            neurion_dot_sanctum_dot_tx__pb2.MsgClaimRewardResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tendermint/types/validator.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'tendermint/types/validator.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from tendermint.crypto import keys_pb2 as tendermint_dot_crypto_dot_keys__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n tendermint/types/validator.proto\x12\x10tendermint.types\x1a\x14gogoproto/gogo.proto\x1a\x1ctendermint/crypto/keys.proto\"\x8a\x01\n\x0cValidatorSet\x12/\n\nvalidators\x18\x01 \x03(\x0b\x32\x1b.tendermint.types.Validator\x12-\n\x08proposer\x18\x02 \x01(\x0b\x32\x1b.tendermint.types.Validator\x12\x1a\n\x12total_voting_power\x18\x03 \x01(\x03\"\x82\x01\n\tValidator\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0c\x12\x33\n\x07pub_key\x18\x02 \x01(\x0b\x32\x1c.tendermint.crypto.PublicKeyB\x04\xc8\xde\x1f\x00\x12\x14\n\x0cvoting_power\x18\x03 \x01(\x03\x12\x19\n\x11proposer_priority\x18\x04 \x01(\x03\"V\n\x0fSimpleValidator\x12-\n\x07pub_key\x18\x01 \x01(\x0b\x32\x1c.tendermint.crypto.PublicKey\x12\x14\n\x0cvoting_power\x18\x02 \x01(\x03\x42\x39Z7github.com/tendermint/tendermint/proto/tendermint/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tendermint.types.validator_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z7github.com/tendermint/tendermint/proto/tendermint/types'
  _globals['_VALIDATOR'].fields_by_name['pub_key']._loaded_options = None
  _globals['_VALIDATOR'].fields_by_name['pub_key']._serialized_options = b'\310\336\037\000'
  _globals['_VALIDATORSET']._serialized_start=107
  _globals['_VALIDATORSET']._serialized_end=245
  _globals['_VALIDATOR']._serialized_start=248
  _globals['_VALIDATOR']._serialized_end=378
  _globals['_SIMPLEVALIDATOR']._serialized_start=380
  _globals['_SIMPLEVALIDATOR']._serialized_end=466
# @@protoc_insertion_point(module_scope)



from google.protobuf.json_format import Parse

from neurionpy.common.rest_client import RestClient
from neurionpy.protos.neurion.fusion.query_pb2 import (
    QueryParamsRequest, QueryParamsResponse,
    QueryGetTaskRequest, QueryGetTaskResponse,
    QueryGetTaskRewardRequest, QueryGetTaskRewardResponse,
    QueryGetCreatorApplicationsRequest, QueryGetCreatorApplicationsResponse,
    QueryGetPendingCreatorApplicationsRequest, QueryGetPendingCreatorApplicationsResponse,
    QueryGetModelsByRoundRequest, QueryGetModelsByRoundResponse,
    QueryGetTaskStakeRequest, QueryGetTaskStakeResponse,
    QueryGetValidationTaskRequest, QueryGetValidationTaskResponse,
    QueryCanTriggerTestingForRoundRequest, QueryCanTriggerTestingForRoundResponse,
    QueryCanStartNewRoundRequest, QueryCanStartNewRoundResponse,
    QueryCanRegisterAsProposerRequest, QueryCanRegisterAsProposerResponse,
    QueryCanRegisterAsValidatorRequest, QueryCanRegisterAsValidatorResponse,
    QueryIsProposerRequest, QueryIsProposerResponse,
    QueryIsValidatorRequest, QueryIsValidatorResponse,
    QueryCanProposeModelRequest, QueryCanProposeModelResponse,
    QueryCanRequestValidationTaskRequest, QueryCanRequestValidationTaskResponse,
    QueryGetPendingValidationTasksRequest, QueryGetPendingValidationTasksResponse,
    QueryCanTerminateRequest, QueryCanTerminateResponse,
    QueryGetScoreDisputeFromUserRequest, QueryGetScoreDisputeFromUserResponse
)
from neurionpy.fusion.interface import FusionQuery


class FusionRestClient(FusionQuery):
    """Fusion REST client implementing all query endpoints."""
    API_URL = "/neurion/fusion"

    def __init__(self, rest_api: RestClient):
        """
        Initialize the Fusion REST client.

        :param rest_api: RestClient instance for making HTTP GET requests.
        """
        self._rest_api = rest_api

    def Params(self, request: QueryParamsRequest) -> QueryParamsResponse:
        """Query module parameters."""
        response = self._rest_api.get(f"{self.API_URL}/params")
        return Parse(response, QueryParamsResponse())

    def GetTask(self, request: QueryGetTaskRequest) -> QueryGetTaskResponse:
        """Query a task by its ID."""
        response = self._rest_api.get(f"{self.API_URL}/get_task/{request.task_id}")
        return Parse(response, QueryGetTaskResponse())

    def GetTaskReward(self, request: QueryGetTaskRewardRequest) -> QueryGetTaskRewardResponse:
        """Query task reward for a given task and user."""
        response = self._rest_api.get(f"{self.API_URL}/get_task_reward/{request.task_id}/{request.user}")
        return Parse(response, QueryGetTaskRewardResponse())

    def GetCreatorApplications(self, request: QueryGetCreatorApplicationsRequest) -> QueryGetCreatorApplicationsResponse:
        """Query creator applications for a given creator."""
        response = self._rest_api.get(f"{self.API_URL}/get_creator_applications/{request.creator}")
        return Parse(response, QueryGetCreatorApplicationsResponse())

    def GetPendingCreatorApplications(self, request: QueryGetPendingCreatorApplicationsRequest) -> QueryGetPendingCreatorApplicationsResponse:
        """Query pending creator applications."""
        response = self._rest_api.get(f"{self.API_URL}/get_pending_creator_applications")
        return Parse(response, QueryGetPendingCreatorApplicationsResponse())

    def GetModelsByRound(self, request: QueryGetModelsByRoundRequest) -> QueryGetModelsByRoundResponse:
        """Query proposed models for a given task and round."""
        response = self._rest_api.get(f"{self.API_URL}/get_models_by_round/{request.task_id}/{request.round}")
        return Parse(response, QueryGetModelsByRoundResponse())

    def GetTaskStake(self, request: QueryGetTaskStakeRequest) -> QueryGetTaskStakeResponse:
        """Query task stake for a given task and user."""
        response = self._rest_api.get(f"{self.API_URL}/get_task_stake/{request.task_id}/{request.user}")
        return Parse(response, QueryGetTaskStakeResponse())

    def GetValidationTask(self, request: QueryGetValidationTaskRequest) -> QueryGetValidationTaskResponse:
        """Query a validation task by its ID."""
        response = self._rest_api.get(f"{self.API_URL}/get_validation_task/{request.id}")
        return Parse(response, QueryGetValidationTaskResponse())

    def CanTriggerTestingForRound(self, request: QueryCanTriggerTestingForRoundRequest) -> QueryCanTriggerTestingForRoundResponse:
        """Check if a user can trigger testing for a specific round."""
        response = self._rest_api.get(f"{self.API_URL}/can_trigger_testing_for_round/{request.user}/{request.task_id}/{request.round}")
        return Parse(response, QueryCanTriggerTestingForRoundResponse())

    def CanStartNewRound(self, request: QueryCanStartNewRoundRequest) -> QueryCanStartNewRoundResponse:
        """Check if a user can start a new round."""
        response = self._rest_api.get(f"{self.API_URL}/can_start_new_round/{request.user}/{request.task_id}/{request.round}")
        return Parse(response, QueryCanStartNewRoundResponse())

    def CanRegisterAsProposer(self, request: QueryCanRegisterAsProposerRequest) -> QueryCanRegisterAsProposerResponse:
        """Check if a user can register as a proposer."""
        response = self._rest_api.get(f"{self.API_URL}/can_register_as_proposer/{request.user}/{request.task_id}")
        return Parse(response, QueryCanRegisterAsProposerResponse())

    def CanRegisterAsValidator(self, request: QueryCanRegisterAsValidatorRequest) -> QueryCanRegisterAsValidatorResponse:
        """Check if a user can register as a validator."""
        response = self._rest_api.get(f"{self.API_URL}/can_register_as_validator/{request.user}/{request.task_id}")
        return Parse(response, QueryCanRegisterAsValidatorResponse())

    def IsProposer(self, request: QueryIsProposerRequest) -> QueryIsProposerResponse:
        """Check if a user is a proposer for a task."""
        response = self._rest_api.get(f"{self.API_URL}/is_proposer/{request.user}/{request.task_id}")
        return Parse(response, QueryIsProposerResponse())

    def IsValidator(self, request: QueryIsValidatorRequest) -> QueryIsValidatorResponse:
        """Check if a user is a validator for a task."""
        response = self._rest_api.get(f"{self.API_URL}/is_validator/{request.user}/{request.task_id}")
        return Parse(response, QueryIsValidatorResponse())

    def CanProposeModel(self, request: QueryCanProposeModelRequest) -> QueryCanProposeModelResponse:
        """Check if a user can propose a model."""
        response = self._rest_api.get(f"{self.API_URL}/can_propose_model/{request.user}/{request.task_id}/{request.round}")
        return Parse(response, QueryCanProposeModelResponse())

    def CanRequestValidationTask(self, request: QueryCanRequestValidationTaskRequest) -> QueryCanRequestValidationTaskResponse:
        """Check if a user can request a validation task."""
        response = self._rest_api.get(f"{self.API_URL}/can_request_validation_task/{request.user}/{request.task_id}/{request.round}")
        return Parse(response, QueryCanRequestValidationTaskResponse())

    def GetPendingValidationTasks(self, request: QueryGetPendingValidationTasksRequest) -> QueryGetPendingValidationTasksResponse:
        """Query pending validation tasks for a user and task."""
        response = self._rest_api.get(f"{self.API_URL}/get_pending_validation_tasks/{request.user}/{request.task_id}")
        return Parse(response, QueryGetPendingValidationTasksResponse())

    def CanTerminate(self, request: QueryCanTerminateRequest) -> QueryCanTerminateResponse:
        """Check if a task can be terminated by the creator."""
        response = self._rest_api.get(f"{self.API_URL}/can_terminate/{request.creator}/{request.task_id}")
        return Parse(response, QueryCanTerminateResponse())

    def GetScoreDisputeFromUser(self, request: QueryGetScoreDisputeFromUserRequest) -> QueryGetScoreDisputeFromUserResponse:
        """Queries a list of score disputes raised by a user."""
        response = self._rest_api.get(f"{self.API_URL}/get_score_dispute_from_user/{request.user}")
        return Parse(response, QueryGetScoreDisputeFromUserResponse())


from gogoproto import gogo_pb2 as _gogo_pb2
from ibc.core.connection.v1 import connection_pb2 as _connection_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GenesisState(_message.Message):
    __slots__ = ("connections", "client_connection_paths", "next_connection_sequence", "params")
    CONNECTIONS_FIELD_NUMBER: _ClassVar[int]
    CLIENT_CONNECTION_PATHS_FIELD_NUMBER: _ClassVar[int]
    NEXT_CONNECTION_SEQUENCE_FIELD_NUMBER: _ClassVar[int]
    PARAMS_FIELD_NUMBER: _ClassVar[int]
    connections: _containers.RepeatedCompositeFieldContainer[_connection_pb2.IdentifiedConnection]
    client_connection_paths: _containers.RepeatedCompositeFieldContainer[_connection_pb2.ConnectionPaths]
    next_connection_sequence: int
    params: _connection_pb2.Params
    def __init__(self, connections: _Optional[_Iterable[_Union[_connection_pb2.IdentifiedConnection, _Mapping]]] = ..., client_connection_paths: _Optional[_Iterable[_Union[_connection_pb2.ConnectionPaths, _Mapping]]] = ..., next_connection_sequence: _Optional[int] = ..., params: _Optional[_Union[_connection_pb2.Params, _Mapping]] = ...) -> None: ...



from gogoproto import gogo_pb2 as _gogo_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class MsgVerifyInvariant(_message.Message):
    __slots__ = ("sender", "invariant_module_name", "invariant_route")
    SENDER_FIELD_NUMBER: _ClassVar[int]
    INVARIANT_MODULE_NAME_FIELD_NUMBER: _ClassVar[int]
    INVARIANT_ROUTE_FIELD_NUMBER: _ClassVar[int]
    sender: str
    invariant_module_name: str
    invariant_route: str
    def __init__(self, sender: _Optional[str] = ..., invariant_module_name: _Optional[str] = ..., invariant_route: _Optional[str] = ...) -> None: ...

class MsgVerifyInvariantResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



from amino import amino_pb2 as _amino_pb2
from gogoproto import gogo_pb2 as _gogo_pb2
from neurion.fusion import params_pb2 as _params_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GenesisState(_message.Message):
    __slots__ = ("params",)
    PARAMS_FIELD_NUMBER: _ClassVar[int]
    params: _params_pb2.Params
    def __init__(self, params: _Optional[_Union[_params_pb2.Params, _Mapping]] = ...) -> None: ...



from cosmos.app.v1alpha1 import config_pb2 as _config_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueryConfigRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class QueryConfigResponse(_message.Message):
    __slots__ = ("config",)
    CONFIG_FIELD_NUMBER: _ClassVar[int]
    config: _config_pb2.Config
    def __init__(self, config: _Optional[_Union[_config_pb2.Config, _Mapping]] = ...) -> None: ...



from abc import ABC, abstractmethod

from neurionpy.protos.cosmos.slashing.v1beta1.query_pb2 import (
    QueryParamsResponse,
    QuerySigningInfoRequest,
    QuerySigningInfoResponse,
    QuerySigningInfosRequest,
    QuerySigningInfosResponse,
)


class Slashing(ABC):
    """Slashing abstract class."""

    @abstractmethod
    def Params(self) -> QueryParamsResponse:
        """
        Params queries the parameters of slashing module.

        :return: QueryParamsResponse
        """

    @abstractmethod
    def SigningInfo(self, request: QuerySigningInfoRequest) -> QuerySigningInfoResponse:
        """
        SigningInfo queries the signing info of given cons address.

        :param request: QuerySigningInfoRequest

        :return: QuerySigningInfoResponse
        """

    @abstractmethod
    def SigningInfos(
        self, request: QuerySigningInfosRequest
    ) -> QuerySigningInfosResponse:
        """
        SigningInfos queries signing info of all validators.

        :param request: QuerySigningInfosRequest

        :return: QuerySigningInfosResponse
        """



from typing import Optional

from neurionpy.protos.neurion.crucible.tx_pb2 import (
    MsgUpdateParams,
    MsgUpdateParamsResponse,
    MsgApplyCreator,
    MsgApplyCreatorResponse,
    MsgApproveApplication,
    MsgApproveApplicationResponse,
    MsgRejectApplication,
    MsgRejectApplicationResponse,
    MsgCreateTask,
    MsgCreateTaskResponse,
    MsgRegisterTrainer,
    MsgRegisterTrainerResponse,
    MsgRegisterScorer,
    MsgRegisterScorerResponse,
    MsgSubmitTrainingResult,
    MsgSubmitTrainingResultResponse,
    MsgStakeToTask,
    MsgStakeToTaskResponse,
    MsgRequestScoringTask,
    MsgRequestScoringTaskResponse,
    MsgSubmitScore,
    MsgSubmitScoreResponse,
    MsgSubmitFinalResult,
    MsgSubmitFinalResultResponse,
    MsgReportModelPlagiarism,
    MsgReportModelPlagiarismResponse,
    MsgAcceptPlagiarismReport,
    MsgAcceptPlagiarismReportResponse,
    MsgRejectPlagiarismReport,
    MsgRejectPlagiarismReportResponse,
    MsgDisputeSubmissionScore,
    MsgDisputeSubmissionScoreResponse,
    MsgStartTask,
    MsgStartTaskResponse,
    MsgAbortTask,
    MsgAbortTaskResponse,
    MsgTriggerTaskToFinalSubmission,
    MsgTriggerTaskToFinalSubmissionResponse,
    MsgTriggerTaskToFinalTesting,
    MsgTriggerTaskToFinalTestingResponse,
    MsgTerminateTask,
    MsgTerminateTaskResponse,
    MsgClaimTaskReward,
    MsgClaimTaskRewardResponse,
    MsgUnstakeFromTask,
    MsgUnstakeFromTaskResponse,
    MsgDisclaimCreatorStatus,
    MsgDisclaimCreatorStatusResponse,
)
from neurionpy.crucible.interface import CrucibleMessage
from neurionpy.synapse.tx import Transaction
from neurionpy.synapse.tx_helpers import SubmittedTx
from neurionpy.synapse.utils import NeurionUtils
from neurionpy.synapse.wallet import Wallet


class CrucibleMsgClient(CrucibleMessage):
    def __init__(self, utils: NeurionUtils, wallet: Optional[Wallet]) -> None:
        """
        Initialize the Crucible message client.

        :param utils: NeurionUtils instance for transaction preparation and broadcasting.
        :param wallet: Wallet instance for signing transactions.
        :raises ValueError: if wallet is not provided.
        """
        if wallet is None:
            raise ValueError("A wallet must be provided to initialize CrucibleMsgClient.")
        self.utils = utils
        self.wallet = wallet

    def _set_creator(self, message) -> None:
        """
        If the message has a 'creator' field and it is not set, assign it to the wallet's address.
        """
        if hasattr(message, "creator") and not bool(message.creator):
            message.creator = str(self.wallet.address())

    def UpdateParams(
        self,
        message: MsgUpdateParams,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Update module parameters.
        Note: This message uses an 'authority' field, so no creator defaulting is applied.
        """
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def ApplyCreator(
        self,
        message: MsgApplyCreator,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Apply to become a creator."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def ApproveApplication(
        self,
        message: MsgApproveApplication,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Approve a creator application."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def RejectApplication(
        self,
        message: MsgRejectApplication,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Reject a creator application."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def CreateTask(
        self,
        message: MsgCreateTask,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Create a new task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def RegisterTrainer(
        self,
        message: MsgRegisterTrainer,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Register as a trainer for a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def RegisterScorer(
        self,
        message: MsgRegisterScorer,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Register as a scorer for a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def SubmitTrainingResult(
        self,
        message: MsgSubmitTrainingResult,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Submit a training result for a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def StakeToTask(
        self,
        message: MsgStakeToTask,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Stake tokens to a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def RequestScoringTask(
        self,
        message: MsgRequestScoringTask,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Request scoring for a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def SubmitScore(
        self,
        message: MsgSubmitScore,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Submit a score for a scoring task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def SubmitFinalResult(
        self,
        message: MsgSubmitFinalResult,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Submit the final result for a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def ReportModelPlagiarism(
        self,
        message: MsgReportModelPlagiarism,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Report model plagiarism."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def AcceptPlagiarismReport(
        self,
        message: MsgAcceptPlagiarismReport,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Accept a plagiarism report."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def RejectPlagiarismReport(
        self,
        message: MsgRejectPlagiarismReport,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Reject a plagiarism report."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def DisputeSubmissionScore(
        self,
        message: MsgDisputeSubmissionScore,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Dispute a submission score."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def StartTask(
        self,
        message: MsgStartTask,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Start a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def AbortTask(
        self,
        message: MsgAbortTask,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Abort a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def TriggerTaskToFinalSubmission(
        self,
        message: MsgTriggerTaskToFinalSubmission,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Trigger task transition to final submission phase."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def TriggerTaskToFinalTesting(
        self,
        message: MsgTriggerTaskToFinalTesting,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Trigger task transition to final testing phase."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def TerminateTask(
        self,
        message: MsgTerminateTask,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Terminate a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def ClaimTaskReward(
        self,
        message: MsgClaimTaskReward,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Claim reward for a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def UnstakeFromTask(
        self,
        message: MsgUnstakeFromTask,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Unstake tokens from a task."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )

    def DisclaimCreatorStatus(
        self,
        message: MsgDisclaimCreatorStatus,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """Disclaim creator status."""
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(
            tx, self.wallet, gas_limit=gas_limit, memo=memo
        )


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: neurion/ganglion/pathway.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'neurion/ganglion/pathway.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eneurion/ganglion/pathway.proto\x12\x10neurion.ganglion\"\xad\x01\n\x07Pathway\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x0f\n\x07\x63reator\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x11\n\tis_public\x18\x05 \x01(\x08\x12\x0c\n\x04ions\x18\x06 \x03(\t\x12\x1e\n\x16\x66\x65\x65_per_thousand_calls\x18\x07 \x01(\x04\x12\x12\n\nfield_maps\x18\x08 \x03(\t\x12\r\n\x05stake\x18\t \x01(\x04\x42\x1aZ\x18neurion/x/ganglion/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'neurion.ganglion.pathway_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\030neurion/x/ganglion/types'
  _globals['_PATHWAY']._serialized_start=53
  _globals['_PATHWAY']._serialized_end=226
# @@protoc_insertion_point(module_scope)



from gogoproto import gogo_pb2 as _gogo_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class MsgCreateClient(_message.Message):
    __slots__ = ("client_state", "consensus_state", "signer")
    CLIENT_STATE_FIELD_NUMBER: _ClassVar[int]
    CONSENSUS_STATE_FIELD_NUMBER: _ClassVar[int]
    SIGNER_FIELD_NUMBER: _ClassVar[int]
    client_state: _any_pb2.Any
    consensus_state: _any_pb2.Any
    signer: str
    def __init__(self, client_state: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., consensus_state: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., signer: _Optional[str] = ...) -> None: ...

class MsgCreateClientResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgUpdateClient(_message.Message):
    __slots__ = ("client_id", "header", "signer")
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    HEADER_FIELD_NUMBER: _ClassVar[int]
    SIGNER_FIELD_NUMBER: _ClassVar[int]
    client_id: str
    header: _any_pb2.Any
    signer: str
    def __init__(self, client_id: _Optional[str] = ..., header: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., signer: _Optional[str] = ...) -> None: ...

class MsgUpdateClientResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgUpgradeClient(_message.Message):
    __slots__ = ("client_id", "client_state", "consensus_state", "proof_upgrade_client", "proof_upgrade_consensus_state", "signer")
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_STATE_FIELD_NUMBER: _ClassVar[int]
    CONSENSUS_STATE_FIELD_NUMBER: _ClassVar[int]
    PROOF_UPGRADE_CLIENT_FIELD_NUMBER: _ClassVar[int]
    PROOF_UPGRADE_CONSENSUS_STATE_FIELD_NUMBER: _ClassVar[int]
    SIGNER_FIELD_NUMBER: _ClassVar[int]
    client_id: str
    client_state: _any_pb2.Any
    consensus_state: _any_pb2.Any
    proof_upgrade_client: bytes
    proof_upgrade_consensus_state: bytes
    signer: str
    def __init__(self, client_id: _Optional[str] = ..., client_state: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., consensus_state: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., proof_upgrade_client: _Optional[bytes] = ..., proof_upgrade_consensus_state: _Optional[bytes] = ..., signer: _Optional[str] = ...) -> None: ...

class MsgUpgradeClientResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgSubmitMisbehaviour(_message.Message):
    __slots__ = ("client_id", "misbehaviour", "signer")
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    MISBEHAVIOUR_FIELD_NUMBER: _ClassVar[int]
    SIGNER_FIELD_NUMBER: _ClassVar[int]
    client_id: str
    misbehaviour: _any_pb2.Any
    signer: str
    def __init__(self, client_id: _Optional[str] = ..., misbehaviour: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., signer: _Optional[str] = ...) -> None: ...

class MsgSubmitMisbehaviourResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



from abc import ABC, abstractmethod

import neurionpy.protos.cosmos.tx.v1beta1.service_pb2 as svc


class TxInterface(ABC):
    """Tx abstract class."""

    @abstractmethod
    def Simulate(self, request: svc.SimulateRequest) -> svc.SimulateResponse:
        """
        Simulate executing a transaction to estimate gas usage.

        :param request: SimulateRequest
        :return: SimulateResponse
        """

    @abstractmethod
    def GetTx(self, request: svc.GetTxRequest) -> svc.GetTxResponse:
        """
        GetTx fetches a tx by hash.

        :param request: GetTxRequest
        :return: GetTxResponse
        """

    @abstractmethod
    def BroadcastTx(self, request: svc.BroadcastTxRequest) -> svc.BroadcastTxResponse:
        """
        BroadcastTx broadcast transaction.

        :param request: BroadcastTxRequest
        :return: BroadcastTxResponse
        """

    @abstractmethod
    def GetTxsEvent(self, request: svc.GetTxsEventRequest) -> svc.GetTxsEventResponse:
        """
        GetTxsEvent fetches txs by event.

        :param request: GetTxsEventRequest
        :return: GetTxsEventResponse
        """



from gogoproto import gogo_pb2 as _gogo_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Proof(_message.Message):
    __slots__ = ("total", "index", "leaf_hash", "aunts")
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    LEAF_HASH_FIELD_NUMBER: _ClassVar[int]
    AUNTS_FIELD_NUMBER: _ClassVar[int]
    total: int
    index: int
    leaf_hash: bytes
    aunts: _containers.RepeatedScalarFieldContainer[bytes]
    def __init__(self, total: _Optional[int] = ..., index: _Optional[int] = ..., leaf_hash: _Optional[bytes] = ..., aunts: _Optional[_Iterable[bytes]] = ...) -> None: ...

class ValueOp(_message.Message):
    __slots__ = ("key", "proof")
    KEY_FIELD_NUMBER: _ClassVar[int]
    PROOF_FIELD_NUMBER: _ClassVar[int]
    key: bytes
    proof: Proof
    def __init__(self, key: _Optional[bytes] = ..., proof: _Optional[_Union[Proof, _Mapping]] = ...) -> None: ...

class DominoOp(_message.Message):
    __slots__ = ("key", "input", "output")
    KEY_FIELD_NUMBER: _ClassVar[int]
    INPUT_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    key: str
    input: str
    output: str
    def __init__(self, key: _Optional[str] = ..., input: _Optional[str] = ..., output: _Optional[str] = ...) -> None: ...

class ProofOp(_message.Message):
    __slots__ = ("type", "key", "data")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    type: str
    key: bytes
    data: bytes
    def __init__(self, type: _Optional[str] = ..., key: _Optional[bytes] = ..., data: _Optional[bytes] = ...) -> None: ...

class ProofOps(_message.Message):
    __slots__ = ("ops",)
    OPS_FIELD_NUMBER: _ClassVar[int]
    ops: _containers.RepeatedCompositeFieldContainer[ProofOp]
    def __init__(self, ops: _Optional[_Iterable[_Union[ProofOp, _Mapping]]] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import any_pb2 as _any_pb2
from gogoproto import gogo_pb2 as _gogo_pb2
from cosmos_proto import cosmos_pb2 as _cosmos_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GenesisState(_message.Message):
    __slots__ = ("authorization",)
    AUTHORIZATION_FIELD_NUMBER: _ClassVar[int]
    authorization: _containers.RepeatedCompositeFieldContainer[GrantAuthorization]
    def __init__(self, authorization: _Optional[_Iterable[_Union[GrantAuthorization, _Mapping]]] = ...) -> None: ...

class GrantAuthorization(_message.Message):
    __slots__ = ("granter", "grantee", "authorization", "expiration")
    GRANTER_FIELD_NUMBER: _ClassVar[int]
    GRANTEE_FIELD_NUMBER: _ClassVar[int]
    AUTHORIZATION_FIELD_NUMBER: _ClassVar[int]
    EXPIRATION_FIELD_NUMBER: _ClassVar[int]
    granter: str
    grantee: str
    authorization: _any_pb2.Any
    expiration: _timestamp_pb2.Timestamp
    def __init__(self, granter: _Optional[str] = ..., grantee: _Optional[str] = ..., authorization: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., expiration: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cometbft/statesync/v1/types.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'cometbft/statesync/v1/types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!cometbft/statesync/v1/types.proto\x12\x15\x63ometbft.statesync.v1\"\x9c\x02\n\x07Message\x12\x44\n\x11snapshots_request\x18\x01 \x01(\x0b\x32\'.cometbft.statesync.v1.SnapshotsRequestH\x00\x12\x46\n\x12snapshots_response\x18\x02 \x01(\x0b\x32(.cometbft.statesync.v1.SnapshotsResponseH\x00\x12<\n\rchunk_request\x18\x03 \x01(\x0b\x32#.cometbft.statesync.v1.ChunkRequestH\x00\x12>\n\x0e\x63hunk_response\x18\x04 \x01(\x0b\x32$.cometbft.statesync.v1.ChunkResponseH\x00\x42\x05\n\x03sum\"\x12\n\x10SnapshotsRequest\"c\n\x11SnapshotsResponse\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\r\x12\x0e\n\x06\x63hunks\x18\x03 \x01(\r\x12\x0c\n\x04hash\x18\x04 \x01(\x0c\x12\x10\n\x08metadata\x18\x05 \x01(\x0c\"=\n\x0c\x43hunkRequest\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\r\x12\r\n\x05index\x18\x03 \x01(\r\"^\n\rChunkResponse\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\r\x12\r\n\x05index\x18\x03 \x01(\r\x12\r\n\x05\x63hunk\x18\x04 \x01(\x0c\x12\x0f\n\x07missing\x18\x05 \x01(\x08\x42\x38Z6github.com/cometbft/cometbft/api/cometbft/statesync/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cometbft.statesync.v1.types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z6github.com/cometbft/cometbft/api/cometbft/statesync/v1'
  _globals['_MESSAGE']._serialized_start=61
  _globals['_MESSAGE']._serialized_end=345
  _globals['_SNAPSHOTSREQUEST']._serialized_start=347
  _globals['_SNAPSHOTSREQUEST']._serialized_end=365
  _globals['_SNAPSHOTSRESPONSE']._serialized_start=367
  _globals['_SNAPSHOTSRESPONSE']._serialized_end=466
  _globals['_CHUNKREQUEST']._serialized_start=468
  _globals['_CHUNKREQUEST']._serialized_end=529
  _globals['_CHUNKRESPONSE']._serialized_start=531
  _globals['_CHUNKRESPONSE']._serialized_end=625
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ibc/applications/interchain_accounts/v1/metadata.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'ibc/applications/interchain_accounts/v1/metadata.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6ibc/applications/interchain_accounts/v1/metadata.proto\x12\'ibc.applications.interchain_accounts.v1\x1a\x14gogoproto/gogo.proto\"\xd1\x01\n\x08Metadata\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x45\n\x18\x63ontroller_connection_id\x18\x02 \x01(\tB#\xf2\xde\x1f\x1fyaml:\"controller_connection_id\"\x12\x39\n\x12host_connection_id\x18\x03 \x01(\tB\x1d\xf2\xde\x1f\x19yaml:\"host_connection_id\"\x12\x0f\n\x07\x61\x64\x64ress\x18\x04 \x01(\t\x12\x10\n\x08\x65ncoding\x18\x05 \x01(\t\x12\x0f\n\x07tx_type\x18\x06 \x01(\tBGZEgithub.com/cosmos/ibc-go/v3/modules/apps/27-interchain-accounts/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ibc.applications.interchain_accounts.v1.metadata_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZEgithub.com/cosmos/ibc-go/v3/modules/apps/27-interchain-accounts/types'
  _globals['_METADATA'].fields_by_name['controller_connection_id']._loaded_options = None
  _globals['_METADATA'].fields_by_name['controller_connection_id']._serialized_options = b'\362\336\037\037yaml:\"controller_connection_id\"'
  _globals['_METADATA'].fields_by_name['host_connection_id']._loaded_options = None
  _globals['_METADATA'].fields_by_name['host_connection_id']._serialized_options = b'\362\336\037\031yaml:\"host_connection_id\"'
  _globals['_METADATA']._serialized_start=122
  _globals['_METADATA']._serialized_end=331
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: neurion/neurion/tx.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'neurion/neurion/tx.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from amino import amino_pb2 as amino_dot_amino__pb2
from cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from neurion.neurion import params_pb2 as neurion_dot_neurion_dot_params__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18neurion/neurion/tx.proto\x12\x0fneurion.neurion\x1a\x11\x61mino/amino.proto\x1a\x17\x63osmos/msg/v1/msg.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x14gogoproto/gogo.proto\x1a\x1cneurion/neurion/params.proto\"\xa8\x01\n\x0fMsgUpdateParams\x12+\n\tauthority\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x32\n\x06params\x18\x02 \x01(\x0b\x32\x17.neurion.neurion.ParamsB\t\xc8\xde\x1f\x00\xa8\xe7\xb0*\x01:4\x82\xe7\xb0*\tauthority\x8a\xe7\xb0*!neurion/x/neurion/MsgUpdateParams\"\x19\n\x17MsgUpdateParamsResponse2h\n\x03Msg\x12Z\n\x0cUpdateParams\x12 .neurion.neurion.MsgUpdateParams\x1a(.neurion.neurion.MsgUpdateParamsResponse\x1a\x05\x80\xe7\xb0*\x01\x42\x19Z\x17neurion/x/neurion/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'neurion.neurion.tx_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\027neurion/x/neurion/types'
  _globals['_MSGUPDATEPARAMS'].fields_by_name['authority']._loaded_options = None
  _globals['_MSGUPDATEPARAMS'].fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _globals['_MSGUPDATEPARAMS'].fields_by_name['params']._loaded_options = None
  _globals['_MSGUPDATEPARAMS'].fields_by_name['params']._serialized_options = b'\310\336\037\000\250\347\260*\001'
  _globals['_MSGUPDATEPARAMS']._loaded_options = None
  _globals['_MSGUPDATEPARAMS']._serialized_options = b'\202\347\260*\tauthority\212\347\260*!neurion/x/neurion/MsgUpdateParams'
  _globals['_MSG']._loaded_options = None
  _globals['_MSG']._serialized_options = b'\200\347\260*\001'
  _globals['_MSGUPDATEPARAMS']._serialized_start=169
  _globals['_MSGUPDATEPARAMS']._serialized_end=337
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_start=339
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_end=364
  _globals['_MSG']._serialized_start=366
  _globals['_MSG']._serialized_end=470
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cosmos/tx/v1beta1/service.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'cosmos/tx/v1beta1/service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from cosmos.base.abci.v1beta1 import abci_pb2 as cosmos_dot_base_dot_abci_dot_v1beta1_dot_abci__pb2
from cosmos.tx.v1beta1 import tx_pb2 as cosmos_dot_tx_dot_v1beta1_dot_tx__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x63osmos/tx/v1beta1/service.proto\x12\x11\x63osmos.tx.v1beta1\x1a\x1cgoogle/api/annotations.proto\x1a#cosmos/base/abci/v1beta1/abci.proto\x1a\x1a\x63osmos/tx/v1beta1/tx.proto\x1a\x14gogoproto/gogo.proto\x1a*cosmos/base/query/v1beta1/pagination.proto\"\x8e\x01\n\x12GetTxsEventRequest\x12\x0e\n\x06\x65vents\x18\x01 \x03(\t\x12:\n\npagination\x18\x02 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest\x12,\n\x08order_by\x18\x03 \x01(\x0e\x32\x1a.cosmos.tx.v1beta1.OrderBy\"\xb2\x01\n\x13GetTxsEventResponse\x12\"\n\x03txs\x18\x01 \x03(\x0b\x32\x15.cosmos.tx.v1beta1.Tx\x12:\n\x0ctx_responses\x18\x02 \x03(\x0b\x32$.cosmos.base.abci.v1beta1.TxResponse\x12;\n\npagination\x18\x03 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"V\n\x12\x42roadcastTxRequest\x12\x10\n\x08tx_bytes\x18\x01 \x01(\x0c\x12.\n\x04mode\x18\x02 \x01(\x0e\x32 .cosmos.tx.v1beta1.BroadcastMode\"P\n\x13\x42roadcastTxResponse\x12\x39\n\x0btx_response\x18\x01 \x01(\x0b\x32$.cosmos.base.abci.v1beta1.TxResponse\"J\n\x0fSimulateRequest\x12%\n\x02tx\x18\x01 \x01(\x0b\x32\x15.cosmos.tx.v1beta1.TxB\x02\x18\x01\x12\x10\n\x08tx_bytes\x18\x02 \x01(\x0c\"y\n\x10SimulateResponse\x12\x33\n\x08gas_info\x18\x01 \x01(\x0b\x32!.cosmos.base.abci.v1beta1.GasInfo\x12\x30\n\x06result\x18\x02 \x01(\x0b\x32 .cosmos.base.abci.v1beta1.Result\"\x1c\n\x0cGetTxRequest\x12\x0c\n\x04hash\x18\x01 \x01(\t\"m\n\rGetTxResponse\x12!\n\x02tx\x18\x01 \x01(\x0b\x32\x15.cosmos.tx.v1beta1.Tx\x12\x39\n\x0btx_response\x18\x02 \x01(\x0b\x32$.cosmos.base.abci.v1beta1.TxResponse*H\n\x07OrderBy\x12\x18\n\x14ORDER_BY_UNSPECIFIED\x10\x00\x12\x10\n\x0cORDER_BY_ASC\x10\x01\x12\x11\n\rORDER_BY_DESC\x10\x02*|\n\rBroadcastMode\x12\x1e\n\x1a\x42ROADCAST_MODE_UNSPECIFIED\x10\x00\x12\x18\n\x14\x42ROADCAST_MODE_BLOCK\x10\x01\x12\x17\n\x13\x42ROADCAST_MODE_SYNC\x10\x02\x12\x18\n\x14\x42ROADCAST_MODE_ASYNC\x10\x03\x32\xf8\x03\n\x07Service\x12{\n\x08Simulate\x12\".cosmos.tx.v1beta1.SimulateRequest\x1a#.cosmos.tx.v1beta1.SimulateResponse\"&\x82\xd3\xe4\x93\x02 \"\x1b/cosmos/tx/v1beta1/simulate:\x01*\x12q\n\x05GetTx\x12\x1f.cosmos.tx.v1beta1.GetTxRequest\x1a .cosmos.tx.v1beta1.GetTxResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/cosmos/tx/v1beta1/txs/{hash}\x12\x7f\n\x0b\x42roadcastTx\x12%.cosmos.tx.v1beta1.BroadcastTxRequest\x1a&.cosmos.tx.v1beta1.BroadcastTxResponse\"!\x82\xd3\xe4\x93\x02\x1b\"\x16/cosmos/tx/v1beta1/txs:\x01*\x12|\n\x0bGetTxsEvent\x12%.cosmos.tx.v1beta1.GetTxsEventRequest\x1a&.cosmos.tx.v1beta1.GetTxsEventResponse\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/cosmos/tx/v1beta1/txsB+Z%github.com/cosmos/cosmos-sdk/types/tx\xc0\xe3\x1e\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.tx.v1beta1.service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z%github.com/cosmos/cosmos-sdk/types/tx\300\343\036\001'
  _globals['_SIMULATEREQUEST'].fields_by_name['tx']._loaded_options = None
  _globals['_SIMULATEREQUEST'].fields_by_name['tx']._serialized_options = b'\030\001'
  _globals['_SERVICE'].methods_by_name['Simulate']._loaded_options = None
  _globals['_SERVICE'].methods_by_name['Simulate']._serialized_options = b'\202\323\344\223\002 \"\033/cosmos/tx/v1beta1/simulate:\001*'
  _globals['_SERVICE'].methods_by_name['GetTx']._loaded_options = None
  _globals['_SERVICE'].methods_by_name['GetTx']._serialized_options = b'\202\323\344\223\002\037\022\035/cosmos/tx/v1beta1/txs/{hash}'
  _globals['_SERVICE'].methods_by_name['BroadcastTx']._loaded_options = None
  _globals['_SERVICE'].methods_by_name['BroadcastTx']._serialized_options = b'\202\323\344\223\002\033\"\026/cosmos/tx/v1beta1/txs:\001*'
  _globals['_SERVICE'].methods_by_name['GetTxsEvent']._loaded_options = None
  _globals['_SERVICE'].methods_by_name['GetTxsEvent']._serialized_options = b'\202\323\344\223\002\030\022\026/cosmos/tx/v1beta1/txs'
  _globals['_ORDERBY']._serialized_start=1051
  _globals['_ORDERBY']._serialized_end=1123
  _globals['_BROADCASTMODE']._serialized_start=1125
  _globals['_BROADCASTMODE']._serialized_end=1249
  _globals['_GETTXSEVENTREQUEST']._serialized_start=216
  _globals['_GETTXSEVENTREQUEST']._serialized_end=358
  _globals['_GETTXSEVENTRESPONSE']._serialized_start=361
  _globals['_GETTXSEVENTRESPONSE']._serialized_end=539
  _globals['_BROADCASTTXREQUEST']._serialized_start=541
  _globals['_BROADCASTTXREQUEST']._serialized_end=627
  _globals['_BROADCASTTXRESPONSE']._serialized_start=629
  _globals['_BROADCASTTXRESPONSE']._serialized_end=709
  _globals['_SIMULATEREQUEST']._serialized_start=711
  _globals['_SIMULATEREQUEST']._serialized_end=785
  _globals['_SIMULATERESPONSE']._serialized_start=787
  _globals['_SIMULATERESPONSE']._serialized_end=908
  _globals['_GETTXREQUEST']._serialized_start=910
  _globals['_GETTXREQUEST']._serialized_end=938
  _globals['_GETTXRESPONSE']._serialized_start=940
  _globals['_GETTXRESPONSE']._serialized_end=1049
  _globals['_SERVICE']._serialized_start=1252
  _globals['_SERVICE']._serialized_end=1756
# @@protoc_insertion_point(module_scope)



from cosmos.base.query.v1beta1 import pagination_pb2 as _pagination_pb2
from gogoproto import gogo_pb2 as _gogo_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.api import annotations_pb2 as _annotations_pb2
from cosmos.auth.v1beta1 import auth_pb2 as _auth_pb2
from cosmos_proto import cosmos_pb2 as _cosmos_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueryAccountsRequest(_message.Message):
    __slots__ = ("pagination",)
    PAGINATION_FIELD_NUMBER: _ClassVar[int]
    pagination: _pagination_pb2.PageRequest
    def __init__(self, pagination: _Optional[_Union[_pagination_pb2.PageRequest, _Mapping]] = ...) -> None: ...

class QueryAccountsResponse(_message.Message):
    __slots__ = ("accounts", "pagination")
    ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    PAGINATION_FIELD_NUMBER: _ClassVar[int]
    accounts: _containers.RepeatedCompositeFieldContainer[_any_pb2.Any]
    pagination: _pagination_pb2.PageResponse
    def __init__(self, accounts: _Optional[_Iterable[_Union[_any_pb2.Any, _Mapping]]] = ..., pagination: _Optional[_Union[_pagination_pb2.PageResponse, _Mapping]] = ...) -> None: ...

class QueryAccountRequest(_message.Message):
    __slots__ = ("address",)
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    address: str
    def __init__(self, address: _Optional[str] = ...) -> None: ...

class QueryAccountResponse(_message.Message):
    __slots__ = ("account",)
    ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    account: _any_pb2.Any
    def __init__(self, account: _Optional[_Union[_any_pb2.Any, _Mapping]] = ...) -> None: ...

class QueryParamsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class QueryParamsResponse(_message.Message):
    __slots__ = ("params",)
    PARAMS_FIELD_NUMBER: _ClassVar[int]
    params: _auth_pb2.Params
    def __init__(self, params: _Optional[_Union[_auth_pb2.Params, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cometbft/services/block_results/v2/block_results.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'cometbft/services/block_results/v2/block_results.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cometbft.abci.v2 import types_pb2 as cometbft_dot_abci_dot_v2_dot_types__pb2
from cometbft.types.v2 import params_pb2 as cometbft_dot_types_dot_v2_dot_params__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6cometbft/services/block_results/v2/block_results.proto\x12\"cometbft.services.block_results.v2\x1a\x1c\x63ometbft/abci/v2/types.proto\x1a\x1e\x63ometbft/types/v2/params.proto\"(\n\x16GetBlockResultsRequest\x12\x0e\n\x06height\x18\x01 \x01(\x03\"\xaa\x02\n\x17GetBlockResultsResponse\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\x32\n\ntx_results\x18\x02 \x03(\x0b\x32\x1e.cometbft.abci.v2.ExecTxResult\x12\x36\n\x15\x66inalize_block_events\x18\x03 \x03(\x0b\x32\x17.cometbft.abci.v2.Event\x12<\n\x11validator_updates\x18\x04 \x03(\x0b\x32!.cometbft.abci.v2.ValidatorUpdate\x12\x43\n\x17\x63onsensus_param_updates\x18\x05 \x01(\x0b\x32\".cometbft.types.v2.ConsensusParams\x12\x10\n\x08\x61pp_hash\x18\x06 \x01(\x0c\x42\x45ZCgithub.com/cometbft/cometbft/api/cometbft/services/block_results/v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cometbft.services.block_results.v2.block_results_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZCgithub.com/cometbft/cometbft/api/cometbft/services/block_results/v2'
  _globals['_GETBLOCKRESULTSREQUEST']._serialized_start=156
  _globals['_GETBLOCKRESULTSREQUEST']._serialized_end=196
  _globals['_GETBLOCKRESULTSRESPONSE']._serialized_start=199
  _globals['_GETBLOCKRESULTSRESPONSE']._serialized_end=497
# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod

from neurionpy.protos.cosmwasm.wasm.v1.query_pb2 import (
    QueryAllContractStateRequest,
    QueryAllContractStateResponse,
    QueryCodeRequest,
    QueryCodeResponse,
    QueryCodesRequest,
    QueryCodesResponse,
    QueryContractHistoryRequest,
    QueryContractHistoryResponse,
    QueryContractInfoRequest,
    QueryContractInfoResponse,
    QueryContractsByCodeRequest,
    QueryContractsByCodeResponse,
    QueryRawContractStateRequest,
    QueryRawContractStateResponse,
    QuerySmartContractStateRequest,
    QuerySmartContractStateResponse,
)


class CosmWasm(ABC):
    """Wasm abstract class."""

    @abstractmethod
    def ContractInfo(
        self, request: QueryContractInfoRequest
    ) -> QueryContractInfoResponse:
        """
        Get the contract meta data.

        :param request: QueryContractInfoRequest

        :return: QueryContractInfoResponse
        """

    @abstractmethod
    def ContractHistory(
        self, request: QueryContractHistoryRequest
    ) -> QueryContractHistoryResponse:
        """
        Get the contract code history.

        :param request: QueryContractHistoryRequest

        :return: QueryContractHistoryResponse
        """

    @abstractmethod
    def ContractsByCode(
        self, request: QueryContractsByCodeRequest
    ) -> QueryContractsByCodeResponse:
        """
        List all smart contracts for a code id.

        :param request: QueryContractsByCodeRequest

        :return: QueryContractsByCodeResponse
        """

    @abstractmethod
    def AllContractState(
        self, request: QueryAllContractStateRequest
    ) -> QueryAllContractStateResponse:
        """
        Get all raw store data for a single contract.

        :param request: QueryAllContractStateRequest

        :return: QueryAllContractStateResponse
        """

    @abstractmethod
    def RawContractState(
        self, request: QueryRawContractStateRequest
    ) -> QueryRawContractStateResponse:
        """
        Get single key from the raw store data of a contract.

        :param request: QueryRawContractStateRequest

        :return: QueryRawContractStateResponse
        """

    @abstractmethod
    def SmartContractState(
        self, request: QuerySmartContractStateRequest
    ) -> QuerySmartContractStateResponse:
        """
        Get smart query result from the contract.

        :param request: QuerySmartContractStateRequest

        :return: QuerySmartContractStateResponse
        """

    @abstractmethod
    def Code(self, request: QueryCodeRequest) -> QueryCodeResponse:
        """
        Get the binary code and metadata for a singe wasm code.

        :param request: QueryCodeRequest

        :return: QueryCodeResponse
        """

    @abstractmethod
    def Codes(self, request: QueryCodesRequest) -> QueryCodesResponse:
        """
        Get the metadata for all stored wasm codes.

        :param request: QueryCodesRequest

        :return: QueryCodesResponse
        """
